import React from "react"
import { Fragment } from "react"
import { Menu, Popover, Transition } from "@headlessui/react"
import { ChevronDownIcon } from "@heroicons/react/solid"
import { classNames } from "../../utils/class-names"

const sortOptions = [
  { name: "Most Popular", href: "#", current: true },
  { name: "Best Rating", href: "#", current: false },
  { name: "Newest", href: "#", current: false },
  { name: "Price: Low to High", href: "#", current: false },
  { name: "Price: High to Low", href: "#", current: false },
]

const ProductFilter = ({ filterables, activeFilters, setActiveFilters }) => {
  const handleChange = e => {
    const { key, value } = JSON.parse(e.target.value)
    const tmp = { ...activeFilters }

    if (e.target.checked) {
      if (!tmp[key]) {
        tmp[key] = []
      }

      tmp[key].push(value)
    } else {
      tmp[key] = tmp[key].filter(item => item !== value)

      if (tmp[key].length === 0) {
        delete tmp[key]
      }
    }

    setActiveFilters(tmp)
  }

  return (
    <section aria-labelledby="filter-heading">
      <h2 id="filter-heading" className="sr-only">
        Filters
      </h2>

      <div className="relative z-10 bg-white border-b border-gray-200 pb-4">
        <div className="max-w-7xl mx-auto px-4 flex items-center justify-between sm:px-6 lg:px-8">
          <Menu as="div" className="relative inline-block text-left">
            <div>
              <Menu.Button className="group inline-flex justify-center text-sm font-medium text-gray-700 hover:text-gray-900">
                Sort
                <ChevronDownIcon
                  className="flex-shrink-0 -mr-1 ml-1 h-5 w-5 text-gray-400 group-hover:text-gray-500"
                  aria-hidden="true"
                />
              </Menu.Button>
            </div>

            <Transition
              as={Fragment}
              enter="transition ease-out duration-100"
              enterFrom="transform opacity-0 scale-95"
              enterTo="transform opacity-100 scale-100"
              leave="transition ease-in duration-75"
              leaveFrom="transform opacity-100 scale-100"
              leaveTo="transform opacity-0 scale-95"
            >
              <Menu.Items className="origin-top-left absolute left-0 mt-2 w-40 rounded-md shadow-2xl bg-white ring-1 ring-black ring-opacity-5 focus:outline-none">
                <div className="py-1">
                  {sortOptions.map(option => (
                    <Menu.Item key={option.name}>
                      {({ active }) => (
                        <a
                          href={option.href}
                          className={classNames(
                            option.current
                              ? "font-medium text-gray-900"
                              : "text-gray-500",
                            active ? "bg-gray-100" : "",
                            "block px-4 py-2 text-sm"
                          )}
                        >
                          {option.name}
                        </a>
                      )}
                    </Menu.Item>
                  ))}
                </div>
              </Menu.Items>
            </Transition>
          </Menu>

          {/* <button
            type="button"
            className="inline-block text-sm font-medium text-gray-700 hover:text-gray-900 sm:hidden"
            onClick={() => setMobileFiltersOpen(true)}
          >
            Filters
          </button> */}

          <div className="hidden sm:block">
            <div className="flow-root">
              <Popover.Group className="-mx-4 flex items-center divide-x divide-gray-200">
                {Object.keys(filterables).map((fKey, keyIds) => (
                  <Popover
                    key={`popover-${fKey}`}
                    className="px-4 relative inline-block text-left"
                  >
                    <Popover.Button className="group inline-flex justify-center text-sm font-medium text-gray-700 hover:text-gray-900">
                      <span>{filterables[fKey].title}</span>
                      {activeFilters &&
                      Object.keys(activeFilters).includes(fKey) ? (
                        <span className="ml-1.5 rounded py-0.5 px-1.5 bg-gray-200 text-xs font-semibold text-gray-700 tabular-nums">
                          {activeFilters[fKey]?.length}
                        </span>
                      ) : null}
                      <ChevronDownIcon
                        className="flex-shrink-0 -mr-1 ml-1 h-5 w-5 text-gray-400 group-hover:text-gray-500"
                        aria-hidden="true"
                      />
                    </Popover.Button>

                    <Transition
                      as={Fragment}
                      enter="transition ease-out duration-100"
                      enterFrom="transform opacity-0 scale-95"
                      enterTo="transform opacity-100 scale-100"
                      leave="transition ease-in duration-75"
                      leaveFrom="transform opacity-100 scale-100"
                      leaveTo="transform opacity-0 scale-95"
                    >
                      <Popover.Panel className="origin-top-right absolute right-0 mt-2 bg-white rounded-md shadow-2xl p-4 ring-1 ring-black ring-opacity-5 focus:outline-none">
                        <form className="space-y-4">
                          {filterables[fKey].values.map((option, optionIdx) => {
                            const checked =
                              activeFilters?.[fKey] &&
                              activeFilters[fKey].includes(option)
                            return (
                              <div
                                key={`filter-item-${fKey}-${optionIdx}`}
                                className="flex items-center"
                              >
                                <input
                                  id={`filter-input-${fKey}-${optionIdx}`}
                                  name={`${optionIdx}[]`}
                                  defaultChecked={checked}
                                  // defaultValue={option}
                                  type="checkbox"
                                  value={JSON.stringify({
                                    key: fKey,
                                    value: option,
                                  })}
                                  onChange={handleChange}
                                  className="h-4 w-4 border-gray-300 rounded text-indigo-600 focus:ring-indigo-500"
                                />
                                <label
                                  htmlFor={`filter-${fKey}-${optionIdx}`}
                                  className="ml-3 pr-6 text-sm font-medium text-gray-900 whitespace-nowrap"
                                >
                                  {option}
                                </label>
                              </div>
                            )
                          })}
                        </form>
                      </Popover.Panel>
                    </Transition>
                  </Popover>
                ))}
              </Popover.Group>
            </div>
          </div>
        </div>
      </div>

      {/* Active filters */}

      <div className="bg-gray-100">
        <div className="max-w-7xl mx-auto py-3 px-4 sm:flex sm:items-center sm:px-6 lg:px-8">
          <h3 className="text-xs font-semibold uppercase tracking-wide text-gray-500">
            Filters
            <span className="sr-only">, active</span>
          </h3>

          <div
            aria-hidden="true"
            className="hidden w-px h-5 bg-gray-300 sm:block sm:ml-4"
          />
          {activeFilters ? (
            <div className="mt-2 sm:mt-0 sm:ml-4">
              <div className="-m-1 flex flex-wrap items-center">
                {Object.keys(activeFilters).map(key => (
                  <div>
                    {activeFilters[key].map((option, optionIdx) => {
                      return (
                        <span
                          key={`active-filter-${key}-${optionIdx}`}
                          className="m-1 inline-flex rounded-full border border-gray-200 items-center py-1.5 pl-3 pr-2 text-sm font-medium bg-white text-gray-900"
                        >
                          <span>{option}</span>
                          <button
                            type="button"
                            className="flex-shrink-0 ml-1 h-4 w-4 p-1 rounded-full inline-flex text-gray-400 hover:bg-gray-200 hover:text-gray-500"
                            onClick={() => {
                              const tmp = { ...activeFilters }
                              tmp[key] = tmp[key].filter(
                                item => item !== option
                              )
                              if (tmp[key].length === 0) {
                                delete tmp[key]
                              }
                              setActiveFilters(tmp)
                            }}
                          >
                            <span className="sr-only">
                              Remove filter for {option}
                            </span>
                            <svg
                              className="h-2 w-2"
                              stroke="currentColor"
                              fill="none"
                              viewBox="0 0 8 8"
                            >
                              <path
                                strokeLinecap="round"
                                strokeWidth="1.5"
                                d="M1 1l6 6m0-6L1 7"
                              />
                            </svg>
                          </button>
                        </span>
                      )
                    })}
                  </div>
                ))}
              </div>
            </div>
          ) : null}
        </div>
      </div>
    </section>
    // <div className="lg:sticky lg:top-0">
    //   {Object.keys(filterables).map((key, index) => {
    //     const filterable = filterables[key]
    //     return (
    //       <div key={index} className="py-4">
    //         <p className="font-medium mb-2">{filterable.title}</p>
    //         <div>
    //           {filterable.values.map((value, index) => {
    //             return (
    //               <label
    //                 key={index}
    //                 className="block py-2 leading-5 text-sm text-gray-700"
    //               >
    //                 <input
    //                   className="checkbox-ui"
    //                   type="checkbox"
    //                   value={JSON.stringify({ key, value })}
    //                   onChange={handleChange}
    //                 />
    //                 {value}
    //               </label>
    //             )
    //           })}
    //         </div>
    //       </div>
    //     )
    //   })}
    // </div>
  )
}

export default ProductFilter
